/** @file
 * @brief Logger class header. This class represents the Logging facility.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#ifndef LOGGER_H
#define LOGGER_H

#include <string>
#include <list>

#include <boost/program_options.hpp>
#include <boost/shared_ptr.hpp>

#include <curl/curl.h>

class Logger
{

private:

    int Loglevel;
    bool Syslog;
    std::string ExternalWarningLog;
    int ExternalWarningLevel;

public:

    typedef boost::shared_ptr<Logger> Ptr;

    Logger();

    ~Logger();

    void log_notice(const std::string& msg) const;

    void log_warning(const std::string& msg, int loglevel) const;

    void log_error(const std::string& msg) const;

    void set_loglevel(const int _loglevel);

    int get_loglevel() const;

    void set_syslog(const bool _syslog);

    bool get_syslog() const;

    void set_log_facility(const int _loglevel, const bool _syslog, const std::string& _external_error_log, const int _external_error_level);

    void print_usage(const boost::shared_ptr<boost::program_options::options_description> opt_desc) const;

    void print_version() const;

    void print_cmd_parsed() const;

    void print_conf_files_parsed() const;

    void print_destructor_call(const std::string& _class) const;

    void print_constructor_call(const std::string& _class) const;

    void print_update_service(const std::string& service) const;

    void print_unknown_cmd_option(const std::string& unknown_option) const;

    void print_unknown_protocol(const std::string& protocol) const;

    void print_load_service_conf(const std::string& filename) const;

    void print_load_main_conf(const std::string& filename) const;

    void print_unknown_service_conf_option(const std::string& service_conf_file, const std::string& unknown_option) const;

    void print_unknown_main_conf_option(const std::string& unknown_option) const;

    void print_error_opening_r(const std::string& filename) const;

    void print_error_opening_rw(const std::string& filename) const;

    void print_error_config_path(const std::string& config_path) const;

    void print_conf_loaded(const std::string& config_path) const;

    void print_conf_not_loaded(const std::string& config_path) const;

    void print_missing_cmd_service_option() const;

    void print_missing_service_conf_option(const std::string& service_conf_file) const;

    void print_runnig_as_daemon(const int pid) const;

    void print_daemon_mode(const bool daemon_mode) const;

    void print_error_fork() const;

    void print_pid_found(const int pid) const;

    void print_process_already_running(const int pid) const;

    void print_caught_sigterm() const;

    void print_caught_siguser1() const;

    void print_caught_sighup() const;

    void print_caught_siguser2() const;

    void print_caught_sigrtmin() const;

    void print_error_setting_signal(const std::string& signal) const;

    void print_init_log_facility() const;

    void print_offline_mode() const;

    void print_serialized_objects_success() const;

    void print_deserialized_objects_success() const;

    void print_service_object(const std::string& message, const std::string& protocol, const std::string& hostname, const std::string& login, const std::string& password, const int update_interval, const int max_updates_within_interval, const int dns_cache_ttl , const std::string& actual_ip, std::list<time_t> lastupdated) const;

    void print_exception_serialize(const std::string& errMsg) const;

    void print_exception_deserialize(const std::string& errMsg) const;

    void print_error_kill_child(const int pid) const;

    void print_child_killed(const int pid) const;

    void print_no_object_file(const std::string& object_file) const;

    void print_hostname(const std::string& hostname) const;

    void print_own_ipv4(const std::string& ip_addr_v4, const std::string& hostname) const;

    void print_own_ipv6(const std::string& ip_addr_v6, const std::string& hostname) const;

    void print_error_hostname_to_ip(const std::string& errMsg, const std::string& hostname) const;

    void print_update_service_successful(const std::string& service) const;

    void print_webcheck_no_ip() const;

    void print_no_wan_ip() const;

    void print_webcheck_url_connection_problem(const char * curl_err_buff, const std::string& url) const;

    void print_webcheck_error(const char * curl_err_buff, const std::string& url) const;

    void print_received_curl_data(const std::string& curl_data) const;

    void print_regex_found_ip(const std::string& ip) const;

    void print_regex_ip_not_found(const std::string& data) const;

    void print_multiple_cmd_option(const std::string& message) const;

    void print_multiple_service_conf_option(const std::string& service_conf_file, const std::string& message) const;

    void print_multiple_main_conf_option(const std::string& main_conf_file, const std::string& message) const;

    void print_update_not_allowed(const time_t current_time, const time_t old_time, const int MaxUpdatesWithinInterval, const std::string& service) const;

    void print_update_service_failure(const std::string& service) const;

    void print_starting_shutdown() const;

    void print_shutdown_succeeded() const;

    void print_shutdown_parent_succeeded() const;

    void print_starting_shutdown_parent() const;

    void print_recheck_dns_entry(const std::string& hostname, const int lastupdated, const int dns_cache_ttl, const int current_time) const;

    void print_missing_cmd_proxy_option() const;

    void print_missing_conf_proxy_option(const std::string& main_conf_filename) const;

    void print_no_domain_part(const std::string& hostname) const;

    void print_curl_error_init(const std::string& err_msg, const CURLcode curl_err_code) const;

    void print_curl_error(const std::string& url, const CURLcode curl_err_code) const;

    void print_curl_error(const std::string& url, const CURLcode curl_err_code, const char * curl_err_buff) const;

    void print_curl_data(const std::string& curl_writedata_buff) const;

    void print_service_not_authorized(const std::string& service, const std::string& username, const std::string& password) const;

    void print_service_not_initialized(const std::string& service) const;

    void print_http_status_code(const std::string& url, const long http_code) const;

    void print_update_failure(const std::string& url, const std::string& curl_data) const;

    void print_update_failure(const std::string& url, const long http_status_code) const;

    void print_invalid_hostname(const std::string& hostname) const;

    void print_ip_is_local(const std::string& ip) const;

    void print_regex_match(const std::string& regex, const std::string& matching_string) const;

    void print_no_regex_match(const std::string& regex, const std::string& not_matching_string) const;

    void print_could_not_parse_received_data(const std::string& curl_data) const;

    void print_could_not_get_initial_gnudip_data() const;

    void print_gnudip_requires_servername() const;

    void print_exception_md5_sum(const std::string& what) const;

    void print_network_error(const std::string& what) const;

    void print_undefined_protocol_error(const std::string& protocol, const std::string& error) const;

    void print_error_external_logging(const std::string& external_prog) const;

    void print_error_parsing_config_file(const std::string& filename, const std::string& error) const;

    void print_error_parsing_cmd(const std::string& error) const;

    void print_webcheck_exceed_interval( const time_t last_webcheck, const int webcheck_interval, const time_t current_time ) const;

    void print_check_service_update(const std::string& hostname, const time_t current_time, const time_t lastupdated) const;

    void print_cached_dns_entry(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_last_update, const std::string& ip_host) const;

    void print_update_service_firttime(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_host) const;

    void print_update_service(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_last_update, const std::string& ip_host, const time_t lastupdated) const;

    void print_update_service_ttl_expired(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_last_update, const std::string& ip_host, const time_t lastupdated, const int dns_cache_ttl, const time_t current_time) const;

    void print_update_service_ttl_not_expired(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_last_update, const std::string& ip_host, const time_t lastupdated, const int dns_cache_ttl, const time_t current_time) const;

    void print_no_update_needed(const std::string& hostname, const std::string& ip_dns_recheck, const std::string& ip_last_update, const std::string& ip_host, const time_t lastupdated) const;

    void print_error_getting_local_wan_ip(const std::string& system_call, const std::string& error) const;

    void print_invalid_service_config() const;
};

#endif
